@@ -1,14 +1,17 @@
 """Test Personal Identifiable Information Transformer using Faker."""
 
+import pickle
+import re
+import tempfile
 from unittest.mock import Mock, call, patch
 
 import numpy as np
 import pandas as pd
 import pytest
 
-from rdt.errors import Error
-from rdt.transformers.null import NullTransformer
-from rdt.transformers.pii.anonymizer import AnonymizedFaker
+from rdt.errors import TransformerInputError, TransformerProcessingError
+from rdt.transformers.categorical import LabelEncoder
+from rdt.transformers.pii.anonymizer import AnonymizedFaker, PseudoAnonymizedFaker
 
 
 class TestAnonymizedFaker:
@@ -67,16 +70,18 @@ class TestAnonymizedFaker:
         )
 
         # Run
-        with pytest.raises(Error, match=expected_message):
+        with pytest.raises(TransformerProcessingError, match=expected_message):
             AnonymizedFaker.check_provider_function('TestProvider', 'TestFunction')
 
-    def test__function(self):
-        """Test that `_function`.
+    def test__function_enforce_uniqueness_false(self):
+        """Test that ``_function`` does not use ``faker.unique``.
 
-        The method `_function` should return a call from the `instance.faker.provider.<function>`.
+        The method ``_function`` should return a call from the
+        ``instance.faker.provider.<function>``.
 
         Mock:
             - Instance of 'AnonymizedFaker'.
+            - Instance ``enforce_uniqueness`` set to `False`
             - Faker instance.
             - A function for the faker instance.
 
@@ -89,8 +94,12 @@ class TestAnonymizedFaker:
         """
         # setup
         instance = Mock()
+        instance.enforce_uniqueness = False
         function = Mock()
+        unique_function = Mock()
         function.return_value = 1
+
+        instance.faker.unique.number = unique_function
         instance.faker.number = function
         instance.function_name = 'number'
         instance.function_kwargs = {'type': 'int'}
@@ -99,9 +108,49 @@ class TestAnonymizedFaker:
         result = AnonymizedFaker._function(instance)
 
         # Assert
+        unique_function.assert_not_called()
         function.assert_called_once_with(type='int')
         assert result == 1
 
+    def test__function_enforce_uniqueness_true(self):
+        """Test that ``_function`` uses the ``faker.unique``.
+
+        The method ``_function`` should return a call from the
+        ``instance.faker.unique.<function>``.
+
+        Mock:
+            - Instance of 'AnonymizedFaker'.
+            - Instance ``enforce_uniqueness`` set to ``True``
+            - Faker instance.
+            - A function for the faker instance.
+
+        Output:
+            - Return value of mocked function.
+
+        Side Effects:
+            - The returned function, when called, has to call the ``faker.unique.<function_name>``
+              function with the provided kwargs.
+        """
+        # setup
+        instance = Mock()
+        instance.enforce_uniqueness = True
+        function = Mock()
+        unique_function = Mock()
+        unique_function.return_value = 1
+
+        instance.faker.unique.number = unique_function
+        instance.faker.number = function
+        instance.function_name = 'number'
+        instance.function_kwargs = {'type': 'int'}
+
+        # Run
+        result = AnonymizedFaker._function(instance)
+
+        # Assert
+        function.assert_not_called()
+        unique_function.assert_called_once_with(type='int')
+        assert result == 1
+
     @patch('rdt.transformers.pii.anonymizer.importlib')
     @patch('rdt.transformers.pii.anonymizer.warnings')
     def test__check_locales(self, mock_warnings, mock_importlib):
@@ -151,8 +200,6 @@ class TestAnonymizedFaker:
             - the ``instance.function_name`` is ``lexify``.
             - the ``instance.locales`` is ``None``.
             - the ``instance.function_kwargs`` is an empty ``dict``.
-            - the ``instance.missing_value_replacement`` is ``None``.
-            - the ``instance.model_missing_values`` is ``False``
             - ``check_provider_function`` has been called once with ``BaseProvider`` and
               ``lexify``.
             - the ``instance._function`` is ``instance.faker.lexify``.
@@ -165,10 +212,9 @@ class TestAnonymizedFaker:
         assert instance.provider_name == 'BaseProvider'
         assert instance.function_name == 'lexify'
         assert instance.function_kwargs == {}
-        assert instance.missing_value_replacement is None
-        assert not instance.model_missing_values
         assert instance.locales is None
         assert mock_faker.Faker.called_once_with(None)
+        assert instance.enforce_uniqueness is False
 
     @patch('rdt.transformers.pii.anonymizer.faker')
     @patch('rdt.transformers.pii.anonymizer.AnonymizedFaker.check_provider_function')
@@ -190,8 +236,6 @@ class TestAnonymizedFaker:
             - the ``instance.function_name`` is ``credit_card_full``.
             - the ``instance.locales`` is ``['en_US', 'fr_FR']``.
             - the ``instance.function_kwargs`` is ``{'type': 'visa'}``.
-            - the ``instance.missing_value_replacement`` is ``None``.
-            - the ``instance.model_missing_values`` is ``False``
             - ``check_provider_function`` has been called once with ``CreditCard`` and
               ``credit_card_full``.
             - the ``instance._function`` is ``instance.faker.credit_card_full``.
@@ -203,7 +247,8 @@ class TestAnonymizedFaker:
             function_kwargs={
                 'type': 'visa'
             },
-            locales=['en_US', 'fr_FR']
+            locales=['en_US', 'fr_FR'],
+            enforce_uniqueness=True
         )
 
         # Assert
@@ -211,10 +256,9 @@ class TestAnonymizedFaker:
         assert instance.provider_name == 'credit_card'
         assert instance.function_name == 'credit_card_full'
         assert instance.function_kwargs == {'type': 'visa'}
-        assert instance.missing_value_replacement is None
-        assert not instance.model_missing_values
         assert instance.locales == ['en_US', 'fr_FR']
         assert mock_faker.Faker.called_once_with(['en_US', 'fr_FR'])
+        assert instance.enforce_uniqueness
 
     def test___init__no_function_name(self):
         """Test the instantiation of the transformer with custom parameters.
@@ -229,95 +273,53 @@ class TestAnonymizedFaker:
             'Please specify the function name to use from the '
             "'credit_card' provider."
         )
-        with pytest.raises(Error, match=expected_message):
+        with pytest.raises(TransformerInputError, match=expected_message):
             AnonymizedFaker(provider_name='credit_card', locales=['en_US', 'fr_FR'])
 
-    def test_get_output_sdtypes(self):
-        """Test the ``get_output_sdtypes``.
-
-        Setup:
-            - initialize a ``AnonymizedFaker`` transformer with default values.
-
-        Output:
-            - the ``output_sdtypes`` returns an empty dictionary.
-        """
+    @patch('rdt.transformers.pii.anonymizer.BaseTransformer.reset_randomization')
+    @patch('rdt.transformers.pii.anonymizer.faker')
+    def test_reset_randomization(self, mock_faker, mock_base_reset):
+        """Test that this function creates a new faker instance."""
         # Setup
-        transformer = AnonymizedFaker()
-        transformer.column_prefix = 'a#b'
+        instance = AnonymizedFaker()
+        instance.locales = ['en_US']
 
         # Run
-        output = transformer.get_output_sdtypes()
+        AnonymizedFaker.reset_randomization(instance)
 
         # Assert
-        expected = {}
-        assert output == expected
+        assert mock_faker.Faker.called_once_with(['en_US'])
+        mock_base_reset.assert_called_once()
 
-    def test_get_output_sdtypes_model_missing_values(self):
-        """Test the ``get_output_sdtypes`` method when a null column is created.
-
-        Setup:
-            - initialize a ``AnonymizedFaker`` transformer which:
-                - sets ``self.null_transformer`` to a ``NullTransformer`` where
-                ``self.model_missing_values`` is True.
-                - sets ``self.column_prefix`` to a string.
-
-        Output:
-            - An ``output_sdtypes`` dictionary is being returned with the ``self.column_prefix``
-              added to the beginning of the keys.
-        """
-        # Setup
-        transformer = AnonymizedFaker()
-        transformer.null_transformer = NullTransformer(missing_value_replacement='fill')
-        transformer.null_transformer._model_missing_values = True
-        transformer.column_prefix = 'a#b'
-
-        # Run
-        output = transformer.get_output_sdtypes()
-
-        # Assert
-        expected = {
-            'a#b.is_null': 'float'
-        }
-        assert output == expected
-
-    @patch('rdt.transformers.pii.anonymizer.NullTransformer')
-    def test__fit(self, mock_null_transformer):
+    def test__fit(self):
         """Test the ``_fit`` method.
 
-        Validate that the ``_fit`` method uses the ``NullTransformer`` to parse the data
-        and learn the length of it.
+        Validate that the ``_fit`` method learns the size of the input data.
 
         Setup:
             - Initialize a ``AnonymizedFaker`` transformer.
-            - Mock the ``NullTransformer``.
 
         Input:
             - ``pd.Series`` containing 3 strings.
 
         Side Effects:
-            - ``NullTransformer`` instance has been created with ``model_missing_values`` as
-              ``False`` and ``missing_value_replacement`` as ``None``.
-            - ``ǸullTransformer`` instance method ``fit`` has been called with the input data.
             - ``instance.data_length`` equals to the length of the input data.
         """
         # Setup
         transformer = AnonymizedFaker()
-
         columns_data = pd.Series(['1', '2', '3'])
 
         # Run
         transformer._fit(columns_data)
 
         # Assert
-        mock_null_transformer.assert_called_once_with(None, False)
-        mock_null_transformer.return_value.fit.called_once_with(columns_data)
         assert transformer.data_length == 3
+        assert transformer.output_properties == {None: {'next_transformer': None}}
 
     def test__transform(self):
         """Test the ``_transform`` method.
 
-        Validate that the ``_transform`` method returns ``None`` when the ``NullTransformer``
-        does not model the missing values.
+        Validate that the ``_transform`` method returns ``None``.
 
         Setup:
             - Initialize a ``AnonymizedFaker`` transformer.
@@ -338,42 +340,6 @@ class TestAnonymizedFaker:
         # Assert
         assert result is None
 
-    def test__transform_model_missing_values(self):
-        """Test the ``_transform`` method.
-
-        Validate that the ``_transform`` method uses the ``NullTransformer`` instance to
-        transform the data.
-
-        Setup:
-            - Initialize a ``AnonymizedFaker`` transformer.
-            - Mock the ``null_transformer`` of the instance.
-            - Mock the return value of the ``null_transformer.transform``.
-
-        Input:
-            - ``pd.Series`` with three values.
-
-        Output:
-            - The second dimension of the mocked return value of the
-              ``null_transformer.transform``.
-        """
-        # Setup
-        columns_data = pd.Series([1, 2, 3])
-        instance = AnonymizedFaker()
-        instance.null_transformer = Mock()
-
-        instance.null_transformer.transform.return_value = np.array([
-            [4, 0],
-            [5, 1],
-            [6, 0],
-        ])
-
-        # Run
-        result = instance._transform(columns_data)
-
-        # Assert
-        instance.null_transformer.transform.assert_called_once_with(columns_data)
-        np.testing.assert_array_equal(result, np.array([0, 1, 0]))
-
     def test__reverse_transform(self):
         """Test the ``_reverse_transform`` method.
 
@@ -382,96 +348,88 @@ class TestAnonymizedFaker:
 
         Setup:
             - Initialize a ``AnonymizedFaker`` transformer.
-            - Mock the ``null_transformer`` of the instance.
-            - Mock the return value of the ``null_transformer.reverse_transform``.
 
         Input:
-            - ``pd.Series`` with three values.
+            - ``None``.
 
         Output:
-            - the output of ``null_transformer.reverse_transform``.
+            - the output a ``numpy.array`` with the generated values from
+              ``instance._function``.
         """
         # Setup
         instance = AnonymizedFaker()
-        instance.null_transformer = Mock()
-        instance.null_transformer.models_missing_values.return_value = False
         instance.data_length = 3
         function = Mock()
         function.side_effect = ['a', 'b', 'c']
 
         instance._function = function
-        instance.null_transformer.reverse_transform.return_value = np.array(['a', 'b', 'c'])
 
         # Run
         result = instance._reverse_transform(None)
 
         # Assert
-        expected_null_call = np.array(['a', 'b', 'c'])
-        null_call = instance.null_transformer.reverse_transform.call_args_list[0][0][0]
-        np.testing.assert_array_equal(null_call, expected_null_call)
         assert function.call_args_list == [call(), call(), call()]
         np.testing.assert_array_equal(result, np.array(['a', 'b', 'c']))
 
-    def test__reverse_transform_models_missing_values(self):
+    def test__reverse_transform_not_enough_unique_values(self):
         """Test the ``_reverse_transform`` method.
 
-        Validate that the ``_reverse_transform`` method uses the ``instance._function``
-        to generate values within the range of the ``instance.data_length``.
+        Test that when calling the ``_reverse_transform`` method and the ``instance._function`` is
+        not generating enough unique values raises an error.
 
         Setup:
-            - Mock the instance of ``AnonymizedFaker``.
-            - Mock the ``instance.null_transformer.reverse_transform`` return value.
+            -Instance of ``AnonymizedFaker``.
 
         Input:
-            - ``pd.DataFrame`` with a column ``is_null`` that contains numeric values.
+            - ``pandas.Series`` representing a column.
 
-        Output:
-            - The mocked return value of the ``null_transformer.reverse_transform``.
-
-        Side Effects:
-            - The ``instance._function`` has been called ``instance.data_length`` with
-              the ``function_args`` as keyword args.
+        Side Effect:
+            - Raises an error.
         """
         # Setup
-        columns_data = pd.DataFrame({
-            'is_null': [0, 1, 0]
-        })
-        instance = Mock()
-        instance.null_transformer.reverse_transform.return_value = pd.Series([
-            'a',
-            np.nan,
-            'c',
-        ])
-        instance.get_output_columns.return_value = ['is_null']
+        instance = AnonymizedFaker('misc', 'boolean', enforce_uniqueness=True)
+        data = pd.Series(['a', 'b', 'c', 'd'])
+        instance.columns = ['a']
+
+        # Run / Assert
+        error_msg = re.escape(
+            'The Faker function you specified is not able to generate 4 unique '
+            "values. Please use a different Faker function for column ('a')."
+        )
+        with pytest.raises(TransformerProcessingError, match=error_msg):
+            instance._reverse_transform(data)
+
+    def test__reverse_transform_size_is_length_of_data(self):
+        """Test the ``_reverse_transform`` method.
+
+        Validate that the ``_reverse_transform`` method calls the ``instance._function`` with
+        the ``instance.function_kwargs`` the ``len(data)`` amount of times.
+
+        Setup:
+            - Initialize a ``AnonymizedFaker`` transformer.
+
+        Input:
+            - ``pd.Series`` with three values.
+
+        Output:
+            - the output a ``numpy.array`` with the generated values from
+              ``instance._function``.
+        """
+        # Setup
+        instance = AnonymizedFaker()
+        data = pd.Series([1, 2, 3])
+        instance.data_length = 0
         function = Mock()
-        function.side_effect = [1, 2, 3]
+        function.side_effect = ['a', 'b', 'c']
+
         instance._function = function
-        instance.data_length = 3
-        instance.function_kwargs = {
-            'type': 'a'
-        }
 
         # Run
-        output = AnonymizedFaker._reverse_transform(instance, columns_data)
+        result = instance._reverse_transform(data)
 
         # Assert
-        expected_output = pd.Series([
-            'a',
-            np.nan,
-            'c'
-        ])
-        pd.testing.assert_series_equal(expected_output, output)
-
-        expected_call = np.array([
-            [1, 0],
-            [2, 1],
-            [3, 0]
-        ])
-        called_arg = instance.null_transformer.reverse_transform.call_args[0][0]
-        np.testing.assert_array_equal(expected_call, called_arg)
-
-        expected_function_calls = [call(), call(), call()]
-        assert function.call_args_list == expected_function_calls
+        assert function.call_args_list == [call(), call(), call()]
+        np.testing.assert_array_equal(result, np.array(['a', 'b', 'c']))
 
     def test___repr__default(self):
         """Test the ``__repr__`` method.
@@ -496,15 +454,299 @@ class TestAnonymizedFaker:
         arguments.
         """
         # Setup
-        instance = AnonymizedFaker('credit_card', 'credit_card_full', model_missing_values=True)
+        instance = AnonymizedFaker('credit_card', 'credit_card_full')
 
         # Run
         res = repr(instance)
 
         # Assert
-        expected_res = (
-            "AnonymizedFaker(provider_name='credit_card', function_name='credit_card_full', "
-            'model_missing_values=True)'
+        expected = "AnonymizedFaker(provider_name='credit_card', function_name='credit_card_full')"
+        assert res == expected
+
+
+class TestPseudoAnonymizedFaker:
+    """Test class for ``PseudoAnonymizedFaker``."""
+
+    @patch('rdt.transformers.pii.anonymizer.warnings')
+    def test___getstate__(self, mock_warnings):
+        """Test that when pickling the warning message is being triggered."""
+        # Setup
+        instance = PseudoAnonymizedFaker()
+
+        expected_warning_msg = (
+            'You are saving the mapping information, which includes the original data. '
+            'Sharing this object with others will also give them access to the original data '
+            'used with this transformer.'
         )
 
-        assert res == expected_res
+        # Run / Assert
+        with tempfile.TemporaryFile() as tmp:
+            pickle.dump(instance, tmp)
+
+        mock_warnings.warn.assert_called_once_with(expected_warning_msg)
+
+    @patch('rdt.transformers.pii.anonymizer.faker')
+    @patch('rdt.transformers.pii.anonymizer.AnonymizedFaker.check_provider_function')
+    def test___init__super_attrs(self, mock_check_provider_function, mock_faker):
+        """Test that initializing an instance is calling properly the ``super`` class.
+
+        Mock:
+            - ``check_provider_function`` mock and assert that this is being called with
+              ``BaseProvider`` and ``lexify``.
+            - ``faker`` mock to ensure that is being called with ``None`` as locales
+              and that the ``lexify`` from the instance is being assigned to the ``_function``.
+
+        Side effects:
+            - the ``instance.provider_name`` is ``BaseProvider``.
+            - the ``instance.function_name`` is ``lexify``.
+            - the ``instance.locales`` is ``None``.
+            - the ``instance.function_kwargs`` is an empty ``dict``.
+            - ``check_provider_function`` has been called once with ``BaseProvider`` and
+              ``lexify``.
+            - the ``instance._function`` is ``instance.faker.lexify``.
+        """
+        # Run
+        instance = PseudoAnonymizedFaker()
+
+        # Assert
+        assert instance._mapping_dict == {}
+        assert instance._reverse_mapping_dict == {}
+
+        # Assert Super Attrs
+        assert instance.provider_name == 'BaseProvider'
+        assert instance.function_name == 'lexify'
+        assert instance.function_kwargs == {}
+        assert instance.locales is None
+        assert mock_faker.Faker.called_once_with(None)
+
+    @patch('rdt.transformers.pii.anonymizer.faker')
+    @patch('rdt.transformers.pii.anonymizer.AnonymizedFaker.check_provider_function')
+    def test___init__custom(self, mock_check_provider_function, mock_faker):
+        """Test the instantiation of the transformer with custom parameters.
+
+        Test that the transformer can be instantiated with a custom provider and function, and
+        this is being assigned properly.
+
+        Mock:
+            - ``check_provider_function`` mock and assert that this is being called with
+              ``CreditCard`` and ``credit_card_full``.
+            - ``faker`` mock to ensure that is being called with ``['en_US', 'fr_FR']`` as locales
+              and that the ``credit_card_full`` from the instance is being assigned to the
+              ``_function``.
+
+        Side effects:
+            - the ``instance.provider_name`` is ``CreditCard``.
+            - the ``instance.function_name`` is ``credit_card_full``.
+            - the ``instance.locales`` is ``['en_US', 'fr_FR']``.
+            - the ``instance.function_kwargs`` is ``{'type': 'visa'}``.
+            - ``check_provider_function`` has been called once with ``CreditCard`` and
+              ``credit_card_full``.
+            - the ``instance._function`` is ``instance.faker.credit_card_full``.
+        """
+        # Run
+        instance = PseudoAnonymizedFaker(
+            provider_name='credit_card',
+            function_name='credit_card_full',
+            function_kwargs={
+                'type': 'visa'
+            },
+            locales=['en_US', 'fr_FR']
+        )
+
+        # Assert
+        assert instance._mapping_dict == {}
+        assert instance._reverse_mapping_dict == {}
+        mock_check_provider_function.assert_called_once_with('credit_card', 'credit_card_full')
+        assert instance.provider_name == 'credit_card'
+        assert instance.function_name == 'credit_card_full'
+        assert instance.function_kwargs == {'type': 'visa'}
+        assert instance.locales == ['en_US', 'fr_FR']
+        assert mock_faker.Faker.called_once_with(['en_US', 'fr_FR'])
+
+    def test_get_mapping(self):
+        """Test the ``get_mapping`` method.
+
+        Validate that the ``get_mapping`` method return a ``deepcopy``  of the
+        ``instance._mapping_dict``.
+
+        Setup:
+            - Instance of ``PseudoAnonymizedFaker``.
+            - ``instance._mapping_dict`` to contain a dictionary.
+
+        Output:
+            - A deepcopy of the ``instance._mapping_dict``.
+        """
+        # Setup
+        instance = PseudoAnonymizedFaker()
+        instance._mapping_dict = {'a': 'b'}
+
+        # Run
+        result = instance.get_mapping()
+
+        # Assert
+        assert result == {'a': 'b'}
+        assert id(result) != id(instance._mapping_dict)
+
+    def test__fit(self):
+        """Test the ``_fit`` method.
+
+        Test that when calling the ``_fit`` method we are populating the ``instance._mapping_dict``
+        with unique values and the ``instance._reverse_mapping_dict`` with those as well.
+
+        Setup:
+            -Instance of ``PseudoAnonymizedFaker``.
+
+        Input:
+            - ``pandas.Series`` representing a column.
+
+        Mock:
+            - Mock the ``instance._function`` to return controlled values.
+
+        Side Effects:
+            - ``instance._mapping_dict`` has been populated with the input unique data as keys and
+              ``_function`` returned values as values.
+            - ``instance._reverse_mapping_dict`` contains the ``_function`` returned values as keys
+              and the input data as values.
+        """
+        # Setup
+        instance = PseudoAnonymizedFaker()
+        instance._function = Mock()
+        instance._function.side_effect = [1, 2, 3]
+        data = pd.Series(['a', 'b', 'c'])
+
+        # Run
+        instance._fit(data)
+
+        # Assert
+        assert instance._mapping_dict == {'a': 1, 'b': 2, 'c': 3}
+        assert instance._reverse_mapping_dict == {1: 'a', 2: 'b', 3: 'c'}
+        assert list(instance.output_properties) == [None]
+        assert list(instance.output_properties[None]) == ['sdtype', 'next_transformer']
+        assert instance.output_properties[None]['sdtype'] == 'categorical'
+
+        transformer = instance.output_properties[None]['next_transformer']
+        assert isinstance(transformer, LabelEncoder)
+        assert transformer.add_noise is True
+
+    def test__fit_not_enough_unique_values_in_faker_function(self):
+        """Test the ``_fit`` method.
+
+        Test that when calling the ``_fit`` method and the ``instance._function`` is not
+        generating enough unique values raises an error.
+
+        Setup:
+            -Instance of ``PseudoAnonymizedFaker``.
+
+        Input:
+            - ``pandas.Series`` representing a column.
+
+        Mock:
+            - Mock the ``instance._function`` to return only 1 value.
+
+        Side Effect:
+            - Raises an error.
+        """
+        # Setup
+        instance = PseudoAnonymizedFaker('misc', 'boolean')
+        data = pd.Series(['a', 'b', 'c', 'd'])
+
+        # Run / Assert
+        error_msg = (
+            'The Faker function you specified is not able to generate '
+            '4 unique values. Please use a different '
+            'Faker function for this column.'
+        )
+        with pytest.raises(TransformerProcessingError, match=error_msg):
+            instance._fit(data)
+
+    def test__transform(self):
+        """Test the ``_transform`` method.
+
+        Test that when ``_transform`` is being performed and no new values are present, the data
+        is being mapped to the ``instance._mapping_dict``.
+
+        Setup:
+            - Instance of ``PseudoAnonymizedTransformer``.
+            - ``_mapping_dict`` to the original data.
+
+        Input:
+            - pandas.Series with some data to be mapped.
+
+        Output:
+            - The ``pandas.Series`` with the ``mapped_data``.
+        """
+        # Setup
+        instance = PseudoAnonymizedFaker()
+        instance._mapping_dict = {'a': 'z', 'b': 'y', 'c': 'x'}
+        instance.columns = ['col']
+
+        data = pd.Series(['a', 'b', 'c'], name='col')
+
+        # Run
+        result = instance._transform(data)
+
+        # Assert
+        pd.testing.assert_series_equal(result, pd.Series(['z', 'y', 'x'], name='col'))
+
+    def test__transform_with_new_values(self):
+        """Test the ``_transform`` method.
+
+        Test that when new data is being passed to the ``_transform``, this raises an error.
+
+        Setup:
+            - Instance of ``PseudoAnonymizedTransformer``.
+            - ``_mapping_dict`` to the original data.
+
+        Input:
+            - pandas.Series with values that are not within the ``_mapping_dict``.
+
+        Side Effects:
+            - Raises a error.
+        """
+        # Setup
+        instance = PseudoAnonymizedFaker()
+        instance._mapping_dict = {'a': 'z', 'b': 'y', 'c': 'x'}
+
+        # Assert / Run
+        error_msg_short = re.escape(
+            'The data you are transforming has new, unexpected values '
+            '(1, 2, 3). Please fit the transformer again using this '
+            'new data.'
+        )
+        error_msg_long = re.escape(
+            'The data you are transforming has new, unexpected values '
+            '(1, 2, 3, 4, 5 and 5 more). Please fit the transformer again using this '
+            'new data.'
+        )
+
+        with pytest.raises(TransformerProcessingError, match=error_msg_short):
+            instance._transform(pd.Series([1, 2, 3]))
+
+        with pytest.raises(TransformerProcessingError, match=error_msg_long):
+            instance._transform(pd.Series([1, 2, 3, 4, 5, 6, 7, 8, 9, 10]))
+
+    def test__reverse_transform(self):
+        """Test the ``_reverse_transform`` method.
+
+        Test that the ``_reverse_transform`` returns the input data.
+
+        Setup:
+            - instance of ``PseudoAnonymizedFaker``.
+
+        Input:
+            - pd.Series
+
+        Output:
+            - The input data.
+        """
+        # Setup
+        instance = PseudoAnonymizedFaker()
+        instance.columns = ['col']
+
+        data = pd.Series(['a', 'b', 'c'], name='col')
+
+        # Run
+        reverse_transformed = instance._reverse_transform(data)
+
+        # Assert
+        pd.testing.assert_series_equal(reverse_transformed, pd.Series(['a', 'b', 'c'], name='col'))
